#include "SDL.h"
#include "SDL_ttf.h"

#define FONT_SIZE 16

void setup_font(TTF_Font *font)
{
  char *familyname = TTF_FontFaceFamilyName(font);
  if(familyname)
    SDL_Log("The family name of the face in the font is: %s", familyname);
  char *stylename = TTF_FontFaceStyleName(font);
  if(stylename)
    SDL_Log("The name of the face in the font is: %s", stylename);
  SDL_Log("The font max height is: %d", TTF_FontHeight(font));
  SDL_Log("The font ascent is: %d", TTF_FontAscent(font));
  SDL_Log("The font descent is: %d", TTF_FontDescent(font));
  SDL_Log("The font line skip is: %d", TTF_FontLineSkip(font));
  if(TTF_FontFaceIsFixedWidth(font))
    SDL_Log("The font is fixed width.");
  else
    SDL_Log("The font is not fixed width.");
  SDL_Log("\n");

  TTF_SetFontStyle(font, TTF_STYLE_BOLD);
}


int main(int argc, char* argv[]) {
  if (argc != 2) {
    fprintf(stderr, "%s [35|default]\n", argv[0]);
    return 1;
  }
  if (strcmp(argv[1], "35") == 0) {
    // FreeType subpixel hinting makes old fonts (Arial) ugly, use old renderer
    putenv("FREETYPE_PROPERTIES=truetype:interpreter-version=35");
  }

  SDL_Init(SDL_INIT_VIDEO);
  SDL_Window *sdlWindow = SDL_CreateWindow("SDL2_ttf test",
					   SDL_WINDOWPOS_UNDEFINED,
					   SDL_WINDOWPOS_UNDEFINED,
					   800, 600,
					   0);
  SDL_Renderer *sdlRenderer = SDL_CreateRenderer(sdlWindow, -1, 0);
  SDL_SetRenderDrawColor(sdlRenderer, 0, 0, 0, 255);
  SDL_RenderClear(sdlRenderer);
  SDL_RenderPresent(sdlRenderer);

  SDL_Surface* screen = SDL_CreateRGBSurface(0, 800,600, 32,
					    0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
  SDL_Texture* sdlTexture = SDL_CreateTexture(sdlRenderer,
					      SDL_PIXELFORMAT_ARGB8888,
					      SDL_TEXTUREACCESS_STREAMING,
					      800, 600);
    
  TTF_Font *system_font = NULL;
  TTF_Font *dialog_font = NULL;
  SDL_RWops* rwops = NULL;
  SDL_Color color = {255, 255, 255};

  TTF_Init();
  SDL_version compile_version;
  const SDL_version *link_version = TTF_Linked_Version();
  SDL_TTF_VERSION(&compile_version);
  SDL_Log("compiled with SDL_ttf version: %d.%d.%d\n", 
	  compile_version.major,
	  compile_version.minor,
	  compile_version.patch);
  SDL_Log("running with SDL_ttf version: %d.%d.%d\n", 
	  link_version->major,
	  link_version->minor,
	  link_version->patch);
  SDL_Log("\n");
  
  rwops = SDL_RWFromFile("/usr/share/wine/fonts/vgasys.fon", "rb");
  if (rwops == NULL) {
      fprintf(stderr, "Please install fonts-wine\n");
      return 1;
  }
  system_font = TTF_OpenFontRW(rwops, 1, FONT_SIZE);
  if (system_font == NULL) {
    fprintf(stderr, "Failed to load 'vgasys.fon': %s", SDL_GetError());
    return 1;
  }
  setup_font(system_font);


  rwops = SDL_RWFromFile("/usr/share/fonts/truetype/liberation/LiberationSans-Regular.ttf", "rb");
  if (rwops == NULL) {
    fprintf(stderr, "Please install fonts-liberation\n");
    return 1;
  }

  dialog_font = TTF_OpenFontRW(rwops, 1, FONT_SIZE);
  if (dialog_font == NULL) {
    fprintf(stderr, "Could not open font 'LiberationSans-Regular.ttf': %s", TTF_GetError());
    return 1;
  }
  setup_font(dialog_font);

  SDL_Surface* tmp = TTF_RenderUTF8_Solid(dialog_font, "This is a test.", color);
  SDL_Log("Text surface height is %d\n", tmp->h);

  int i = 0;
  for (int y = 0; y < screen->h; y += tmp->h) {
    i++;
    char text[1024];
    sprintf(text, "%02d: vgasys.fon", i);
    SDL_Surface* rendered_text = TTF_RenderUTF8_Solid(system_font, text, color);
    SDL_Rect dest = { 000, y, rendered_text->w, rendered_text->h };
    SDL_BlitSurface(rendered_text, NULL, screen, &dest);
  }

  int j = 0;
  for (int y = 0; y < screen->h; y += tmp->h) {
    j++;
    char text[1024];
    sprintf(text, "%02d: LiberationSans-Regular.ttf", j);
    SDL_Surface* rendered_text = TTF_RenderUTF8_Solid(dialog_font, text, color);
    SDL_Rect dest = { 400, y, rendered_text->w, rendered_text->h };
    SDL_BlitSurface(rendered_text, NULL, screen, &dest);
  }

  char filename[1024];
  char* env;
  env = getenv("FREETYPE_PROPERTIES");
  if (env == NULL) env = "default";
  sprintf(filename, "test-%s-%d.%d.%d.bmp",
	  env,
	  link_version->major,
	  link_version->minor,
	  link_version->patch);
  SDL_SaveBMP(screen, filename);

  while (1) {
    SDL_UpdateTexture(sdlTexture, NULL, screen->pixels, 800 * sizeof (Uint32));
    SDL_RenderCopy(sdlRenderer, sdlTexture, NULL, NULL);
    SDL_RenderPresent(sdlRenderer);
    SDL_Event ev;
    SDL_WaitEvent(&ev);
    if (ev.type == SDL_QUIT)
      break;
  }

  TTF_CloseFont(dialog_font);
  TTF_CloseFont(system_font);
  TTF_Quit();
  SDL_Quit();
}

/**
 * Local Variables:
 * compile-command: "gcc $(pkg-config SDL2_ttf --cflags --libs) test.c"
 * End:
 */
